﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace TROM
{
    [StaticConstructorOnStartup]
    public class Shelling_HighExplosive : Bombardment
    {
        public float impactAreaRadius = 15f;

        public new FloatRange explosionRadiusRange = new FloatRange(3f, 6f);

        public int randomFireRadius = 25;

        public int bombIntervalTicks = 18;

        public int warmupTicks = 60;

        public int explosionCount = 20;

        private int ticksToNextEffect;

        private IntVec3 nextExplosionCell = IntVec3.Invalid;

        private List<BombardmentProjectile> projectiles = new List<BombardmentProjectile>();
        private BombardmentProjectile projectile = new BombardmentProjectile();

        public const int EffectiveAreaRadius = 23;

        private const int StartRandomFireEveryTicks = 20;

        private const int EffectDuration = 60;

        private static readonly Material ProjectileMaterial = MaterialPool.MatFrom("Things/Projectile/Bullet_Big", ShaderDatabase.Transparent, Color.white);

        protected int TicksLeft => duration - TicksPassed;

        bool done = false;

        public static readonly SimpleCurve DistanceChanceFactor = new SimpleCurve
        {
        new CurvePoint(0f, 1f),
        new CurvePoint(1f, 0.1f)
        };

        public override void SpawnSetup(Map map, bool respawningAfterReload)
        {
            base.explosionCount = 1;

            base.SpawnSetup(map, respawningAfterReload);
            if (!respawningAfterReload)
            {
                GetNextExplosionCell();
            }
        }

        public override void Tick()
        {
            if (base.Destroyed)
            {
                return;
            }
            if (warmupTicks > 0)
            {
                warmupTicks--;
                if (warmupTicks == 0)
                {
                    StartStrike();
                }
            }
            else
            {
                if (done) base.Tick();
                if (Find.TickManager.TicksGame % 20 == 0 && TicksLeft > 0)
                {
                    StartRandomFire();
                }
            }
            EffectTick();
        }

        private void EffectTick()
        {
            if (!nextExplosionCell.IsValid)
            {
                ticksToNextEffect = warmupTicks - bombIntervalTicks;
                GetNextExplosionCell();
            }
            ticksToNextEffect--;
            if (ticksToNextEffect <= 0 && TicksLeft >= bombIntervalTicks)
            {
                if (!done)
                {
                    SoundDefOf.Bombardment_PreImpact.PlayOneShot(new TargetInfo(nextExplosionCell, base.Map));
                    projectiles.Add(new BombardmentProjectile(60, nextExplosionCell));
                    ticksToNextEffect = bombIntervalTicks;
                    GetNextExplosionCell();
                    done = true;
                }
            }
            for (int num = projectiles.Count - 1; num >= 0; num--)
            {
                projectiles[num].Tick();
                if (projectiles[num].LifeTime <= 0)
                {
                    TryDoExplosion(projectiles[num]);
                    projectiles.RemoveAt(num);
                }
            }
        }

        private void TryDoExplosion(BombardmentProjectile proj)
        {
            List<Thing> list = base.Map.listerThings.ThingsInGroup(ThingRequestGroup.ProjectileInterceptor);
            for (int i = 0; i < list.Count; i++)
            {
                if (list[i].TryGetComp<CompProjectileInterceptor>().CheckBombardmentIntercept(this, proj))
                {
                    return;
                }
            }
            float explosionRadius = explosionRadiusRange.RandomInRange;
            TryDestroyRoof(base.Map, proj.targetCell, explosionRadius);
            GenExplosion.DoExplosion(proj.targetCell, base.Map, explosionRadius, DamageDefOf.Bomb, instigator, -1, -1f, null, projectile: def, weapon: weaponDef);
        }

        public override void Draw()
        {
            base.Draw();
            if (!projectiles.NullOrEmpty())
            {
                for (int i = 0; i < projectiles.Count; i++)
                {
                    projectiles[i].Draw(ProjectileMaterial);
                }
            }
        }

        private void StartRandomFire()
        {
            IntVec3 intVec = (from x in GenRadial.RadialCellsAround(base.Position, randomFireRadius, useCenter: true)
                              where x.InBounds(base.Map)
                              select x).RandomElementByWeight((IntVec3 x) => DistanceChanceFactor.Evaluate(x.DistanceTo(base.Position)));
            List<Thing> list = base.Map.listerThings.ThingsInGroup(ThingRequestGroup.ProjectileInterceptor);
            for (int i = 0; i < list.Count; i++)
            {
                if (!list[i].TryGetComp<CompProjectileInterceptor>().BombardmentCanStartFireAt(this, intVec))
                {
                    return;
                }
            }
            FireUtility.TryStartFireIn(intVec, base.Map, Rand.Range(0.1f, 0.925f));
        }

        private void GetNextExplosionCell()
        {
            nextExplosionCell = (from x in GenRadial.RadialCellsAround(base.Position, impactAreaRadius, useCenter: true)
                                 where x.InBounds(base.Map)
                                 select x).RandomElementByWeight((IntVec3 x) => DistanceChanceFactor.Evaluate(x.DistanceTo(base.Position) / impactAreaRadius));
        }

        private void TryDestroyRoof(Map map, IntVec3 pos, float explosionRadius)
        {
            for (int i = -1; i < 2; i++)
            {
                for (int j = -1; j < 2; j++)
                {
                    bool wall = false;
                    if (map.thingGrid.CellContains(new IntVec3(pos.x + i, 0, pos.z + j), ThingDefOf.Wall)) wall = true;
                    RoofCollapserImmediate.DropRoofInCells(new IntVec3(pos.x + i, 0, pos.z + j), map);
                    if (wall && Rand.Bool) GenSpawn.Spawn(ThingDefOf.CollapsedRocks, new IntVec3(pos.x + i, 0, pos.z + j), map);
                }
            }
        }

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref impactAreaRadius, "impactAreaRadius", 15f);
            Scribe_Values.Look(ref explosionRadiusRange, "explosionRadiusRange", new FloatRange(6f, 8f));
            Scribe_Values.Look(ref randomFireRadius, "randomFireRadius", 25);
            Scribe_Values.Look(ref bombIntervalTicks, "bombIntervalTicks", 18);
            Scribe_Values.Look(ref warmupTicks, "warmupTicks", 0);
            Scribe_Values.Look(ref ticksToNextEffect, "ticksToNextEffect", 0);
            Scribe_Values.Look(ref nextExplosionCell, "nextExplosionCell");
            Scribe_Collections.Look(ref projectiles, "projectiles", LookMode.Deep);
            if (Scribe.mode == LoadSaveMode.PostLoadInit)
            {
                if (!nextExplosionCell.IsValid)
                {
                    GetNextExplosionCell();
                }
                if (projectiles == null)
                {
                    projectiles = new List<BombardmentProjectile>();
                }
            }
        }
    }
}